@@ -1,13 +1,13 @@
 import base64
 import getpass
 import os
-from typing import Optional
+from typing import Optional, List
 
 from cryptography.fernet import Fernet, InvalidToken
 from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
 
-from isisdl.backend.utils import User, config, error_text, path, logger
-from isisdl.settings import password_hash_algorithm, password_hash_length, password_hash_iterations, env_var_name_username, env_var_name_password, is_autorun, master_password, database_file_location
+from isisdl.utils import User, config, logger
+from isisdl.settings import password_hash_algorithm, password_hash_length, password_hash_iterations, env_var_name_username, env_var_name_password, is_autorun, master_password, error_text
 
 last_password: Optional[str] = None
 last_username: Optional[str] = None
@@ -39,6 +39,7 @@ def decryptor(password: str, content: str) -> Optional[str]:
 
     try:
         decrypted = Fernet(key).decrypt(content.encode()).decode()
+
         # Only cache the entry if it got successfully decrypted
         global last_password
         last_password = password
@@ -57,37 +58,56 @@ def store_user(user: User, password: Optional[str] = None) -> None:
     config.password_encrypted = bool(password)
 
 
-def get_credentials() -> User:
+def get_credentials(alternative_master_passwords: Optional[List[str]] = None) -> User:
     """
     Prioritizes:
         Cached Password > Environment variable > Database > Input
     """
     global last_username
+    alternative_master_passwords = alternative_master_passwords or []
+
+    def prompt_user() -> User:
+        print("Please provide authentication for ISIS.")
+        username = input("Username: ")
+        password = getpass.getpass("Password: ")
+        logger.set_username(username)
+
+        return User(username, password)
 
     # First check the environment variables for username *and* password
     env_username, env_password = os.getenv(env_var_name_username), os.getenv(env_var_name_password)
     if env_username is not None and env_password is not None:
         return User(env_username, env_password)
 
-    # Now query the database
-    user_store_encrypted = config.password_encrypted
-
+    # Now try the database
     if config.username is not None and config.password is not None:
-        if not user_store_encrypted:
-            decrypted = decryptor(master_password, config.password)
-            if decrypted is None:
-                print(f"{error_text}\nI could not decrypt the password even though it is set to be encrypted with the master password.\n"
-                      f"You can delete the file `{path(database_file_location)}` and store your password again!")
-                exit(1)
+        if not config.password_encrypted:
+            for _password in [master_password] + alternative_master_passwords:
+                decrypted = decryptor(_password, config.password)
+                if decrypted is not None:
+                    break
+
+            else:
+                print(f"""
+{error_text} I could not decrypt the password even though it is set to be encrypted
+with the master password. This probably means that the master password
+changed since you saved your password.
+
+You can either enter your password manually or rerun me with `isisdl --init` to re-store your password.
+""")
+
+                return prompt_user()
 
             return User(config.username, decrypted)
 
         while True:
+            # Maybe use the cache
             if last_password is not None:
                 possible_actual_password = decryptor(last_password, config.password)
                 if possible_actual_password is not None:
                     return User(config.username, possible_actual_password)
 
+            # Prompt for a password
             user_password = getpass.getpass("Please enter the passphrase: ")
             actual_password = decryptor(user_password, config.password)
             if actual_password is None:
@@ -99,11 +119,4 @@ def get_credentials() -> User:
         exit(1)
 
     # If nothing is found prompt the user
-    print("Please provide authentication for ISIS.")
-    username = input("Username: ")
-    password = getpass.getpass("Password: ")
-    logger.set_username(username)
-
-    return User(username, password)
-
-# TODO: Generate a random key on a per-user system. Will be stored in `.state.db` ?
+    return prompt_user()
