@@ -7,7 +7,6 @@ import time
 from base64 import standard_b64decode
 from collections import defaultdict
 from concurrent.futures import ThreadPoolExecutor
-from dataclasses import dataclass, field
 from datetime import datetime
 from email.utils import parsedate_to_datetime
 from itertools import repeat, chain
@@ -34,7 +33,9 @@ from isisdl.utils import calculate_local_checksum
 class SessionWithKey(Session):
     key: str
     token: str
-    _blocks: DefaultDict[str, Queue[None]] = defaultdict(Queue)
+
+    __slots__ = tuple(__annotations__)  # type: ignore
+
     _lock = Lock()
 
     def __init__(self, key: str, token: str):
@@ -145,8 +146,10 @@ class PreMediaContainer:
     is_cached: bool
     parent_path: Path
 
+    __slots__ = tuple(__annotations__)  # type: ignore
+
     def __init__(self, url: str, course: Course, media_type: MediaType, name: Optional[str] = None, relative_location: Optional[str] = None, size: Optional[int] = None, time: Optional[int] = None):
-        relative_location = (relative_location or "").strip("/")
+        relative_location = (relative_location or media_type.dir_name).strip("/")
         if config.make_subdirs is False:
             relative_location = ""
 
@@ -174,7 +177,6 @@ class PreMediaContainer:
         return self._name is not None and self.time is not None and self.size is not None
 
 
-@dataclass
 class MediaContainer:
     _name: str
     url: str
@@ -184,11 +186,29 @@ class MediaContainer:
     course: Course
     media_type: MediaType
     size: int
-    checksum: Optional[str] = None
-    current_size: Optional[int] = None
-    _stop: bool = False
-    _links: List[MediaContainer] = field(default_factory=list)
-    _done: bool = False
+    _links: List[MediaContainer]
+    checksum: Optional[str]
+    current_size: Optional[int]
+    _stop: bool
+    _done: bool
+
+    __slots__ = tuple(__annotations__)  # type: ignore
+
+    def __init__(self, _name: str, url: str, download_url: str, path: Path, time: int, course: Course, media_type: MediaType, size: int,
+                 checksum: Optional[str] = None, _links: Optional[List[MediaContainer]] = None) -> None:
+        self._name = _name
+        self.url = url
+        self.download_url = download_url
+        self.path = path
+        self.time = time
+        self.course = course
+        self.media_type = media_type
+        self.size = size
+        self.checksum = checksum
+        self.current_size = None
+        self._stop = False
+        self._links = _links or []
+        self._done = False
 
     @classmethod
     def from_dump(cls, url: str, course: Course) -> Union[bool, MediaContainer]:
@@ -405,7 +425,7 @@ class MediaContainer:
             return False
 
         acc = True
-        for attr in self.__dict__:
+        for attr in self.__slots__:
             if attr in {"current_size", "_links", "_done"}:
                 continue
 
@@ -511,13 +531,20 @@ class MediaContainer:
         self._done = True
 
 
-@dataclass
 class Course:
     old_name: str
     _name: str
     name: str
     course_id: int
 
+    __slots__ = tuple(__annotations__)  # type: ignore
+
+    def __init__(self, old_name: str, _name: str, name: str, course_id: int) -> None:
+        self.old_name = old_name
+        self._name = _name
+        self.name = name
+        self.course_id = course_id
+
     @classmethod
     def from_dict(cls, info: Dict[str, Any]) -> Course:
         old_name = cast(str, info["displayname"])
@@ -865,6 +892,7 @@ def check_for_conflicts_in_files(files: List[MediaContainer]) -> List[MediaConta
     for file in {file.path: file for file in files}.values():
         hard_link_conflicts[f"{file.course.course_id} {file._name} {file.size}"].append(file)
 
+    # TODO: More conflict: If two files have the same download url (across courses), hard link them.
     for conflict in hard_link_conflicts.values():
         if len(conflict) != 1:
             conflict.sort(key=lambda x: x.time)
@@ -897,7 +925,7 @@ def check_for_conflicts_in_files(files: List[MediaContainer]) -> List[MediaConta
                 final_list.append(item)
 
         else:
-            logger.assert_fail(f"conflict: {[item.__dict__ for item in conflict]}")
+            logger.assert_fail(f"conflict: {[{x: getattr(item, x) for x in item.__slots__} for item in conflict]}")
             continue
 
     return final_list
